public class Harmonizer
{
    1000::ms => dur updateRate;

    // Initialize the harmonizer UGens
    SndBuf2 bufs[12];
    Gain individualGainL[12];
    Gain individualGainR[12];
    Gain masterL => NRev revL => dac.left;
    Gain masterR => NRev revR => dac.right;
    revL.mix(0.05);
    revR.mix(0.05);

    float vol[12];

    Step unity;
    Envelope individualGainEnvelope[12];
    ["C ", "C#", "D ", "D#", "E ", "F ", "F#", "G ", "G#", "A ", "A#", "B "] @=> string pitchNames[];

    // Connect the Harmonizer
    for (0 => int i; i < 12; i++)
    {
        // Set the frequency of the individual oscillators, C4 to B4
        //bufs[i].freq(Std.mtof(i + 60));
        me.dir() + "/harmonies/harmony" + (i+1) + ".wav" => bufs[i].read;
        1 => bufs[i].loop;
        // Set the individual gain UGens to 0
        0 => individualGainL[i].gain;
        0 => individualGainR[i].gain;
        // Set all Envelopes to update at the same rate
        individualGainEnvelope[i].duration(updateRate);

        // Signal chain from buf to master
        bufs[i].chan(0) => individualGainL[i] => masterL;
        bufs[i].chan(1) => individualGainR[i] => masterR;
        // Envelope chain
        unity => individualGainEnvelope[i] => blackhole;
    }

    fun void updateHarmonyKey(float chroma[])
    {
        -1 => int maxIndex;
        0 => float maxValue;
        // Get max value of the chroma array
        for (0 => int i; i < 12; i++)
        {
            if (chroma[i] > maxValue)
            {
                i => maxIndex;
                chroma[i] => maxValue;
            }
        }

        // Set the gain of each individual gain envelope
        // 0 if not the max value
        // 1 if the max value
        for (0 => int i; i < 12; i++)
        {
            individualGainEnvelope[i].target(i == maxIndex ? 1 : 0);
            //<<< pitchNames[i], individualGainEnvelope[i].target() >>>;
        }
    }

    fun void updateHarmonizer()
    {
        while (true)
        {
            for (0 => int i; i < 12; i++)
            {
                individualGainL[i].gain( individualGainEnvelope[i].last() );
                individualGainR[i].gain( individualGainEnvelope[i].last() );
                individualGainL[i].gain() => vol[i];
            }

            1::samp => now;
        }
    }

    spork ~ updateHarmonizer();
}

