public class JingleBells
{
    90 => global int bpm;
    60000/bpm => global int beat;
    0 => global int count;
    32 => global int totalBeats;

    // Loop UGens
    SndBuf loops[4];
    Gain loopGain[4];
    Gain master => dac;

    // Jingle UGen
    SndBuf jingle => master => dac;
    me.dir() + "/jinglebells/jingle.wav" => jingle.read;
    1 => jingle.loop;

    // Loop change envelope UGens
    Step unity;
    Envelope loopGainEnvelope[4];
    (beat/4)::ms => dur updateRate;

    // Key rules
    0 => global int verseOrChorus; // 0 for verse, 1 for chorus
    [1, 1, 1, 1, 1, 2, 2, 1, 1, 2, 2, 3] @=> int verseLoopMap[];
    [1, 1, 3, 3, 1, 2, 2, 3, 3, 1, 1, 1] @=> int chorusLoopMap[];

    // Initialize Jingle Bells
    for (0 => int i; i < 4; i++)
    {
        // Initialize the SndBufs
        me.dir() + "/jinglebells/loop-" + i + ".wav" => loops[i].read;
        1 => loops[i].loop;
        // Set loop[i] gain to 0
        0 => loopGain[i].gain;
        // Initialize the loopGainEnvelopes
        loopGainEnvelope[i].duration(updateRate);

        // Loop Signal Chain
        loops[i] => loopGain[i] => master;
        // Envelope Signal Chain
        unity => loopGainEnvelope[i] => blackhole;
    }

    // Default start loop
    0 => global int loopIndex; // Which loop to play
    loopGainEnvelope[loopIndex].target(1);

    fun void updateJingle(float chroma[])
    {
        -1 => int maxIndex;
        0 => float maxValue;
        // Get max value of the chroma array
        for (0 => int i; i < 12; i++)
        {
            if (chroma[i] > maxValue)
            {
                i => maxIndex;
                chroma[i] => maxValue;
            }
        }

        // If no new chroma value
        // or if it isn't beat 1 or 3
        if (maxIndex == -1 || count % 2 == 0) 
        { 
            return;
        }

        // Continue
        // Chroma to key loop mapping
        // Which loop to play
        verseOrChorus ? chorusLoopMap[maxIndex] : verseLoopMap[maxIndex] => loopIndex;

        // Update the loop gain envelopes based on loopIndex
        for (0 => int i; i < 4; i++)
        {
            loopGainEnvelope[i].target(i == loopIndex ? 1 : 0);
        }
    }

    fun void updateProJingle(int choice)
    {
        // Update the loop gain envelopes based on loopIndex
        for (0 => int i; i < 4; i++)
        {
            loopGainEnvelope[i].target(i == choice ? 1 : 0);
        }
        choice => loopIndex;
    }

    fun void processJingle()
    {
        while (true)
        {
            for (0 => int i; i < 4; i++)
            {
                loopGain[i].gain( loopGainEnvelope[i].last() );
            }

            1::samp => now;
        }
    }

    spork ~ processJingle();

    fun void play()
    {
        while (true)
        {
            count++;
            beat::ms => now;
            if (count == totalBeats) { 0 => count; }
        }
    }

    spork ~ play();
}
